#ifndef MATRICE_HPP
#define MATRICE_HPP

#include <cmath>
#include <iostream>
#include <valarray>
#include <vector>
using namespace std;

template <class K,int n, int m> class matrice{
    protected:
    valarray<K> M; // Justifier Valarray
    int index(int i, int j){return i*m+j; }
    public:
    matrice(){
        M.resize(n*m);
    }
    matrice (const matrice & A){
        M=A.M;
    }
    K get_aij(int i, int j);
    void set_aij(int i, int j, K x);
    matrice operator+(matrice A);
    matrice operator*(double);
    matrice operator-(matrice A);
    matrice & operator = (const matrice &);
    template<class Kc,int nc, int mc,int pc> 
    matrice<Kc,nc,pc> operator*(matrice<Kc,mc,pc>);
};

template<class K,int n, int m>  matrice<K,n,m> matrice<K,n,m>::operator*(double lambda){
    matrice<K,n,m> res;
    for( int i=0;i<n;i++){
        for( int j=0;j<m;j++){
            res.set_aij(i,j,this->get_aij(i,j)*lambda);
        }
    }
    return res;
}

template<class K,int n, int m>  K matrice<K,n,m>::get_aij(int i, int j){
    return M[this->index(i,j)];
}

template<class K,int n, int m>  void matrice<K,n,m>::set_aij(int i, int j, K x){
    M[this->index(i,j)]=x;
}

template<class K,int n, int m>  matrice<K,n,m> matrice<K,n,m>::operator+(matrice<K,n,m> A){
    matrice<K,n,m> res;
    res.M= this->M + A.M;
    return res;
}

template<class K,int n, int m>  matrice<K,n,m> matrice<K,n,m>::operator-(matrice<K,n,m> A){
    matrice<K,n,m> res;
    res.M= this->M - A.M;
    return res;
}


template <class K,int n, int m> matrice<K,n,m> & matrice<K,n,m>::operator = (const matrice<K,n,m> & A){
    if (this != &A){
        this->M=A.M;
    }
    return *this;
}


// fonction externe à la classe calculant la transposé d'une matrice
template<class K,int n, int m>  matrice<K,m,n> transpose( matrice<K,n,m> A ){
    matrice<K,m,n> res;
    for (int i =0;i <m;i++){
        for (int j =0;j <n;j++){
            res.set_aij(i,j,A.get_aij(j,i));
        }
    }
    return res;
}
// fonction externe à la classe calculant la le produit de deux matrices
template<class K,int n, int m,int p>  matrice<K,n,p> matrice<K,n,m>::operator*( matrice<K,m,p> B){
    matrice<K,n,p> res;
    K loc;
    for (int i =0;i <n;i++){
        for (int j =0;j <p;j++){
            loc=this->get_aij(i,0)*B.get_aij(0,j);
            for (int k =1;k <m;k++){
                loc= loc+this->get_aij(i,k)*B.get_aij(k,j);
            }
            res.set_aij(i,j,loc);
        }
    }
    return res;
    
}


#endif
