#ifndef REGRESSION_HPP
#define REGRESSION_HPP

#include <cmath>
#include <iostream>
#include <valarray>
#include <vector>
#include "matrice.hpp"
#include "regularisation.hpp"

using namespace std;

template <class K,int n, int d> class Regression{
    protected:
    matrice<K,n,n> A;
    matrice<K,n,1> b;
    matrice<K,n,1> sol;
    K (*kernel) (int, valarray<K> , valarray<K>);
    regularisation<K,n> * r;
    public:
    Regression (){ 
        kernel=NULL; 
    }
    Regression ( K (*pk) (int, valarray<K> , valarray<K>), regularisation<K,n> * reg){ 
        kernel=pk; 
        r = reg;
    }
    Regression (const Regression & reg){
        A= reg.A;
        b =reg.b;
        r= reg.r;
    }
    void set_A(int i, int j, valarray<K> xi, valarray<K> xj);
    void set_b(int i, K yi);
    K get_sol(int i);
    matrice<K,n,1> gradient();
    void gradient_descente(int nmax, K lambda);

};
template <class K,int n,int d> K Regression<K,n,d>::get_sol(int i){
    return sol.get_aij(i,0);
 }

 template <class K,int n,int d> void Regression<K,n,d>::set_A(int i, int j, valarray<K> xi, valarray<K> xj){
     A.set_aij(i,j,this->kernel(d,xi,xj));
 }

 template <class K,int n, int d> void Regression<K,n,d>::set_b(int i, K yi){
     b.set_aij(i,0,yi);
 }

template <class K,int n, int d> matrice<K,n,1> Regression<K,n,d>::gradient(){
      matrice<K,n,n> prod1;
      matrice<K,n,1> prod2;
      matrice<K,n,1> grad;
      prod1= produit(A,transpose(A)); 
      prod2= produit(A,b); 
      grad= produit(prod1, sol) - prod2 + r->derivate_reg_term(sol);
      return grad;
}

template <class K,int n,int d> void Regression<K,n,d>::gradient_descente(int nmax, K lambda){
    int niter=0;
    K error;
    matrice<K,n,1> grad;
    while (niter< nmax){
        grad = this->gradient();
        this->sol = this->sol - grad*lambda;
        niter++;
    }
}
#endif
