#ifndef REGULARISATION_HPP
#define REGULARISATION_HPP

#include <cmath>
#include <iostream>
#include <valarray>
#include <vector>
#include "matrice.hpp"

using namespace std;

template <class K, int d> class regularisation{
    public:
    virtual K reg_term(matrice<K,d,1> x)=0;
    virtual matrice<K,d,1> derivate_reg_term(matrice<K,d,1> x)=0;
};

template <class K, int d> class ridge: public regularisation<K,d>{
    protected:
        double alpha; 
    public:
     ridge(){
        alpha=0.0;
    }
    ridge(K al){
        alpha=al;
    }

    K reg_term(matrice<K,d,1> x){
        K norm2= x.get_aij(0,0)*x.get_aij(0,0);
        for( int i=1;i<d;i++){
            norm2= norm2+x.get_aij(i,0)*x.get_aij(i,0);
        }
        return alpha*norm2;
    }
    matrice<K,d,1> derivate_reg_term(matrice<K,d,1> x){
        matrice<K,d,1> res;
        for( int i=0;i<d;i++){
            res.set_aij(i,0,x.get_aij(i,0)*alpha);
        }
        return res; 
    }
};

template <class K, int d> class lasso: public regularisation<K,d>{
    protected:
        double alpha; 
    public:
     lasso(){
        alpha=0.0;
    }
    lasso(K al){
        alpha=al;
    }

    K reg_term(matrice<K,d,1> x){
        K norm1= abs(x.get_aij(0,0));
        for( int i=1;i<d;i++){
            norm1= norm1+abs(x.get_aij(i,0));
        }
        return alpha*norm1;
    }
    matrice<K,d,1> derivate_reg_term(matrice<K,d,1> x){
        matrice<K,d,1> res;
        K loc;
        for( int i=0;i<d;i++){
            loc=0.0;
            if (x.get_aij(i,0)>0.0){
                res.set_aij(i,0,alpha);
            }
            else{
                res.set_aij(i,0,-alpha);
            }
        }
        return res; 
    }
};

#endif
